def test_docker_show_run_options():
    pass


def test_docker_create():
    pass


def test_docker_start():
    pass


def test_docker_stop():
    pass


def test_docker_restart():
    pass


def test_docker_recreate():
    pass


def test_docker_kill():
    pass


def test_docker_rm():
    pass


def test_docker_logs():
    pass


def test_docker_inspect():
    pass


def test_docker_attach():
    pass


def test_docker_exec():
    pass


def test_docker_shell():
    pass


def test_docker_root_shell():
    pass


def test_docker_pull():
    pass


def test_docker_build():
    pass
